#!/usr/bin/env wish

#******************************
# 
# ospuzzle
# Version 2014-10-07
# Copyright (C) 2012-2014 Oliver Scholl < tcltk@posteo.de >
#
# Requirement:
# tcl 8.4 or higher
# tk 8.4 or higher
#
# Recommendation:
# imagemagick
# 
# Beschreibung des Programms:
# Das Programm lädt eine Bilddatei, zerteilt das Bild in
# rechteckige Puzzlestücke (dabei bleibt die Ecke rechts
# unten leer) und verschiebt die Puzzlestücke zufällig. 
# Der Anwender verschiebt nun solange die Puzzle-Stücke
# bis das Bild wieder vollständig ist.
#
# This program is free software; you can redistribute it and/or 
# modify it under the terms of the GNU General Public License as 
# published by the Free Software Foundation; either version 2 of 
# the license, or any later version.
#
# This program is distributed in the hope that it will be useful, 
# but WITHOUT ANY WARRANTY; without even the implied warranty of 
# merchantability or fitness for a particular purpose. 
# See the GNU General Public License for mor details.
#
#*************************************

package require msgcat

set ProgrammOrdner ".osProgramme"
set ProgrammName "osPuzzle"
set ProgrammVersion "2014-10-07"
set ProgrammTitel "Puzzle"

# Debian Package:
#set Lokalisation [file join / usr share simplixpuzzle] ; # Directory of the localization files. Please adapt to Debian needs.

set Konf(Skriptname) [info script]
if {[file type $Konf(Skriptname)] == "link"} {
	set Konf(Skriptname) [file readlink $Konf(Skriptname)]
}

set Konf(Ordner) [file join $env(HOME) $ProgrammOrdner $ProgrammName]
if {[file exist $Konf(Ordner)] == 0} {
	file mkdir $Konf(Ordner)
}

set Konf(Datei) [file join $Konf(Ordner) $ProgrammName.conf] ; # config file
set Puzzle(SpielstandDatei) [file join $Konf(Ordner) $ProgrammName.dat] ; # Directory where to save the game
set Puzzle(BildName) [file join $Konf(Ordner) $ProgrammName.ppm] ; # Directory where to save the photo
set InitialordnerBildOeffnen [file join $env(HOME)] ; # Initial directory when the user loads a new photo
set Lokalisation [file join [file dirname $Konf(Skriptname)]] ; # Ordner mit den Sprachdateien

#::msgcat::mclocale "en"
::msgcat::mcload [file join [file dirname $Konf(Skriptname)]]

set Konf(Sprache) English ; # Sprache (Default-Wert)
set Konf(Breite) 300 ; # Breite des Puzzles (Default-Wert)
set Konf(ZeileMax) 4 ; # Anzahl der Zeilen im Puzzle (Default-Wert)
set Konf(SpalteMax) 4 ; # Anzahl der Spalten im Puzzle (Default-Wert)
set Konf(VerschiebenMax) 12 ; # Anzahl der Verschiebungen der Puzzle-Teile (Schwierigkeitsgrad)  (Default-Wert)
set Konf(ImageMagick) 0 ; # 1, wenn das Bild mit ImageMagick erstellt werden soll (Default-Wert). Sonst 0.

set Puzzle(Aktiv) 0 ; # 0, wenn noch kein Puzzle aktiv ist. Sonst 1.
set Puzzle(AnzahlZuege) 0 ; # Zählt die Züge, die der User benötigt

set Feld(1,1) "" ; # muss Dummy-artig als Array definiert werden
set ButtonNameLeeresFeld "" ; # Name des leeren Felds

# ***** Prozeduren, die keine andere Prozeduren aufrufen *****

proc SpielstandSpeichern {} {
	global Feld
	global Puzzle

	set f [open $Puzzle(SpielstandDatei) w]
	puts $f "$Puzzle(ZeileMax)"
	puts $f "$Puzzle(SpalteMax)"
	puts $f "$Puzzle(AnzahlZuege)"
	foreach i [array names Feld] {
		puts $f "$i/$Feld($i)"
	}
	close $f
}

proc SpielstandLaden {} {
	global Feld
	global Puzzle

	# Prüfen, ob die Dateien existieren
	if {[glob -nocomplain -types f $Puzzle(SpielstandDatei)] eq ""} {return}
	if {[glob -nocomplain -types f $Puzzle(BildName)] eq ""} {return}

	# Datei einlesen
	set f [open $Puzzle(SpielstandDatei) r]
	gets $f Puzzle(ZeileMax)
	gets $f Puzzle(SpalteMax)
	gets $f Puzzle(AnzahlZuege)

	while {[gets $f Zeile]>=0} {
		set Stelle [string first "/" $Zeile]
		set i [string range $Zeile 0 [expr $Stelle-1]] 
		set Wert [string range $Zeile [expr $Stelle+1] end] 
		set Feld($i) $Wert
	}
	close $f
	
	# Puzzle azeigen
	set Puzzle(Aktiv) 1
	PuzzleAufbauen 0
}

proc KonfigurationSpeichern {} {
	global Konf

	set f [open $Konf(Datei) w]
	puts $f "<Language>$Konf(Sprache)"
	puts $f "<Width>$Konf(Breite)"
	puts $f "<Lines>$Konf(ZeileMax)"
	puts $f "<Rows>$Konf(SpalteMax)"
	puts $f "<Moves>$Konf(VerschiebenMax)"
	puts $f "<ImageMagick>$Konf(ImageMagick)"
	close $f
}

proc KonfigurationEinlesen {} {
	global Konf

	# Konf(Datei) einlesen
	if {[glob -nocomplain -types f $Konf(Datei)] eq ""} {return}
	set f [open $Konf(Datei) r]
	while {[gets $f Zeile] >= 0} {
		if {[string range $Zeile 0 9] eq "<Language>"} {
			set Konf(Sprache) [string range $Zeile 10 end]
		}
		if {[string range $Zeile 0 6] eq "<Width>"} {
			set Konf(Breite) [string range $Zeile 7 end]
		}
		if {[string range $Zeile 0 6] eq "<Lines>"} {
			set Konf(ZeileMax) [string range $Zeile 7 end]
		}
		if {[string range $Zeile 0 5] eq "<Rows>"} {
			set Konf(SpalteMax) [string range $Zeile 6 end]
		}
		if {[string range $Zeile 0 6] eq "<Moves>"} {
			set Konf(VerschiebenMax) [string range $Zeile 7 end]
		}
		if {[string range $Zeile 0 12] eq "<ImageMagick>"} {
			set Konf(ImageMagick) [string range $Zeile 13 end]
		}
	}
	close $f
}

proc SpracheEinstellen {} {
	global Konf
	global Lokalisation

	switch $Konf(Sprache) {
		Deutsch {set SpracheKurzform de}
		English {set SpracheKurzform en}
		Espanol {set SpracheKurzform es}
		Italiano {set SpracheKurzform it}
		Türkce {set SpracheKurzform tr}
		default {set Konf(Sprache) English; set SpracheKurzform en}
	}
	::msgcat::mclocale $SpracheKurzform
	::msgcat::mcload $Lokalisation
}

proc DialogModal {Titel Text {Fokus 1} {Button1 "OK"} {Button2 ""} {Button3 ""}} {
	# Erzeugt einen Modal-Dialog mit ein bis drei Buttons
	# Rückgabewert: die Nummer des gedrückten Buttons (1 bis 3)
	# Titel = Titel des Dialogs
	# Text = Text, der im Modal-Dialog angezeigt wird
	# Fokus = Eine Zahl 1 bis 3. Setzt den Fokus auf einen der Buttons
	# Button1 bis Button3 = Beschriftung der Buttons
	
	global DialogModalReturn
	
	set DialogModalReturn 0
	set ButtonText(1) $Button1
	set ButtonText(2) $Button2
	set ButtonText(3) $Button3
	
	toplevel .modalDlg
	label .modalDlg.lb -text $Text -justify left -wraplength 5i
	pack .modalDlg.lb -side top -pady 3 -padx 3

	set Position left
	for {set i 1} {$i<=3} {incr i} {
		if {$ButtonText($i) ne ""} {
			button .modalDlg.bt$i -text $ButtonText($i) -command "set DialogModalReturn $i;destroy .modalDlg"
			bind .modalDlg.bt$i <KeyPress-Return> ".modalDlg.bt$i invoke"
			pack .modalDlg.bt$i -side $Position -pady 3 -padx 3 -expand 1
		}
	}
	focus .modalDlg.bt$Fokus
		
	wm title .modalDlg $Titel
	wm transient .modalDlg .
	wm resizable .modalDlg 0 0

	tkwait visibility .modalDlg
	grab set .modalDlg
	tkwait window .modalDlg
	return $DialogModalReturn
}

proc DialogSpielEinstellungen {} {

	# Zeigt einen Dialog, in dem der Anwender die Spieleinstellungen
	# (Puzzlebreite, Puzzlehöhe, Anzahl Zeilen, Anzahl Spalten usw.)
	# eingeben kann.
	
	global Konf
	global Dlg

	set Dlg(Breite) $Konf(Breite) ; # Breite des Puzzles
	set Dlg(ZeileMax) $Konf(ZeileMax) ; # Anzahl der Zeilen im Puzzle
	set Dlg(SpalteMax) $Konf(SpalteMax) ; # Anzahl der Spalten im Puzzle
	set Dlg(VerschiebenMax) $Konf(VerschiebenMax) ; # Anzahl der Verschiebungen der Puzzle-Teile
	set Dlg(ImageMagick) $Konf(ImageMagick) ; # 1, wenn das Bild mit ImageMagick erstellt werden soll. Sonst 0.

	toplevel .einstellungenDlg
	label .einstellungenDlg.lbBreite -text [::msgcat::mc "Breite (Pixel):"] -justify left
	label .einstellungenDlg.lbPuzzle(ZeileMax) -text [::msgcat::mc "Zeilen:"] -justify left
	label .einstellungenDlg.lbPuzzle(SpalteMax) -text [::msgcat::mc "Spalten:"] -justify left
	label .einstellungenDlg.lbVerschiebenMax -text [::msgcat::mc "Schwierigkeit:"] -justify left

	entry .einstellungenDlg.enBreite -width 5 -textvariable Dlg(Breite)
	entry .einstellungenDlg.enPuzzle(ZeileMax) -width 5 -textvariable Dlg(ZeileMax)
	entry .einstellungenDlg.enPuzzle(SpalteMax) -width 5 -textvariable Dlg(SpalteMax)
	entry .einstellungenDlg.enVerschiebenMax -width 5 -textvariable Dlg(VerschiebenMax)

	checkbutton .einstellungenDlg.cbImageMagick -text [::msgcat::mc "Benutze ImageMagick"] -anchor w -variable Dlg(ImageMagick)
	label .einstellungenDlg.lbHinweis -text [::msgcat::mc "Hinweis: Wenn die Breite leer bleibt, wird das Bild in Originalgröße gezeigt. Für eine bessere Bildqualität sollte ImageMagick installiert und genutzt werden."] -justify left -wraplength 3i
	
	frame .einstellungenDlg.f1
	button .einstellungenDlg.f1.btOK -text [::msgcat::mc "OK"] -command {set Konf(Breite) $Dlg(Breite);set Konf(ZeileMax) $Dlg(ZeileMax);set Konf(SpalteMax) $Dlg(SpalteMax);set Konf(VerschiebenMax) $Dlg(VerschiebenMax);set Konf(ImageMagick) $Dlg(ImageMagick);destroy .einstellungenDlg}
	button .einstellungenDlg.f1.btAbbruch -text [::msgcat::mc "Abbruch"] -command "destroy .einstellungenDlg"
	bind .einstellungenDlg.f1.btOK <KeyPress-Return> ".einstellungenDlg.f1.btOK invoke"
	bind .einstellungenDlg.f1.btAbbruch <KeyPress-Return> ".einstellungenDlg.f1.btAbbruch invoke"

	grid .einstellungenDlg.lbPuzzle(ZeileMax) -row 0 -column 0 -pady 3 -padx 3 -sticky w
	grid .einstellungenDlg.enPuzzle(ZeileMax) -row 0 -column 1 -pady 3 -padx 3 -sticky w
	grid .einstellungenDlg.lbPuzzle(SpalteMax) -row 1 -column 0 -pady 3 -padx 3 -sticky w
	grid .einstellungenDlg.enPuzzle(SpalteMax) -row 1 -column 1 -pady 3 -padx 3 -sticky w
	grid .einstellungenDlg.lbVerschiebenMax -row 2 -column 0 -pady 3 -padx 3 -sticky w
	grid .einstellungenDlg.enVerschiebenMax -row 2 -column 1 -pady 3 -padx 3 -sticky w
	grid .einstellungenDlg.lbBreite -row 3 -column 0 -pady 3 -padx 3 -sticky w
	grid .einstellungenDlg.enBreite -row 3 -column 1 -pady 3 -padx 3 -sticky w
	grid .einstellungenDlg.cbImageMagick -row 4 -column 0 -pady 3 -padx 3 -sticky w
	grid .einstellungenDlg.lbHinweis -row 5 -column 0 -pady 3 -padx 3 -columnspan 2 -sticky w
	grid .einstellungenDlg.f1 -row 6 -column 0 -columnspan 2 -pady 3 -sticky nsew
	pack .einstellungenDlg.f1.btOK -side left -expand 1
	pack .einstellungenDlg.f1.btAbbruch -side right -expand 1
	focus .einstellungenDlg.f1.btOK

	set FensterBreite [winfo reqwidth .einstellungenDlg]
	set FensterHoehe [winfo reqheight .einstellungenDlg]
	set FensterX [expr {([winfo vrootwidth .]-$FensterBreite)/2}]
	set FensterY [expr {([winfo vrootheight .]-$FensterHoehe)/2}]
	#wm geometry .einstellungenDlg ${FensterBreite}x${FensterHoehe}+${FensterX}+${FensterY}
	wm geometry .einstellungenDlg +${FensterX}+${FensterY}
				
	wm title .einstellungenDlg [::msgcat::mc "Einstellungen"]
	wm transient .einstellungenDlg .
	wm resizable .einstellungenDlg 0 0

	tkwait visibility .einstellungenDlg
	grab set .einstellungenDlg
	tkwait window .einstellungenDlg
}

proc BildAktualisieren {Pruefen} {
	# Zeigt die Puzzlestücke in der aktuellen Anordnung an
	# und prüft, ob das Bild fertig gepuzzelt ist.
	
	global Feld 
	global Puzzle
	
	# Puzzlestücke anzeigen
	for {set Zeile 1} {$Zeile<=$Puzzle(ZeileMax)} {incr Zeile} {
		for {set Spalte 1} {$Spalte<=$Puzzle(SpalteMax)} {incr Spalte} {
			set ButtonName $Feld(${Zeile},${Spalte})
			#puts $Zeile,$Spalte,$ButtonName
			grid .f1.b$ButtonName -row [expr $Zeile-1] -column [expr $Spalte-1] -sticky nsew 
		}
	}
	
	# Prüfen, ob das Puzzle fertig ist
	if {$Pruefen == 1} {
		set Fertig 1
		foreach i [array names Feld] {
			set Trenner [string last , $i]
			set Zeile [string range $i 0 [expr $Trenner-1]]
			set Spalte [string range $i [expr $Trenner+1] end]
			set ButtonName ${Zeile}_$Spalte
			#puts "Fertig: i=$i Feld=$Feld($i) Button=$ButtonName Ergebnis=$Fertig"
			if {$Feld($i) ne $ButtonName} {
				set Fertig 0
			}
		}
		if {$Fertig == 1} {
			set Titel [::msgcat::mc "Ende"]
			set Text [::msgcat::mc "Klasse. Das Bild ist fertig. Du hast %d Züge gebraucht." $Puzzle(AnzahlZuege)]
			DialogModal $Titel $Text 1 "OK"
			# Rahmenlinien ausschalten
			for {set Zeile 1} {$Zeile<=$Puzzle(ZeileMax)} {incr Zeile} {
				for {set Spalte 1} {$Spalte<=$Puzzle(SpalteMax)} {incr Spalte} {
					set ButtonName ${Zeile}_${Spalte}
					.f1.b$ButtonName config -borderwidth 0			
				}
			}
		}
	}
}

# ***** Prozeduren, die andere Prozeduren aufrufen *****

wm protocol . WM_DELETE_WINDOW {
	# wird beim Schliessen des Hauptfensters ausgeführt
	ProgrammBeenden
}

proc ProgrammBeenden {} {
	# Die Prozedur wird beim Schliessen des Hauptfensters ausgeführt.
	# Speichert die Konfiguration und den aktuellen Spielstand.
	# Beendet danach die Anwendung.
	
	global Puzzle
	
	KonfigurationSpeichern
	if {$Puzzle(Aktiv) == 1} {
		SpielstandSpeichern
	}
	destroy .
}

proc SpracheAendern {} {
	SpracheEinstellen
	set Titel [::msgcat::mc "Hinweis"]
	set Text [::msgcat::mc "Die Änderung wird erst nach einem Neustart des Programms wirksam."]
	DialogModal $Titel $Text
}

proc InfoAnzeigen {ProgrammName ProgrammVersion} {
	set Titel [::msgcat::mc "Info"]
	set Text "${ProgrammName}\nVersion ${ProgrammVersion}\n\nCopyright (C) 2012-2014 Oliver Scholl\nwww.tcltk.info\n
This program is free software; you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation; either version 2 of the license, or any later version.\n
This software is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of merchantability or fitness for a particular purpose. See the GNU General Public License for mor details."
	DialogModal $Titel $Text 1
}

proc ButtonKlick {Button} {
	# Verschiebt die Puzzlestücke, wenn der Anwender auf ein
	# Puzzlestück klickt. Dabei wird die Anzahl der Züge gezählt.
	
	global Feld 
	global Puzzle
	global ButtonNameLeeresFeld
	
	set Trenner [string last b $Button]
	set ButtonName [string range $Button [expr $Trenner+1] end]
	foreach i [array names Feld] {
		if {$Feld($i) eq $ButtonName} {
			set Trenner [string last , $i]
			set Zeile [string range $i 0 [expr $Trenner-1]]
			set Spalte [string range $i [expr $Trenner+1] end]
		}
		if {$Feld($i) eq $ButtonNameLeeresFeld} {
			set Trenner [string last , $i]
			set ZeileLeeresFeld [string range $i 0 [expr $Trenner-1]]
			set SpalteLeeresFeld [string range $i [expr $Trenner+1] end]
		}
	}
	if {($Zeile == $ZeileLeeresFeld) && ($Spalte == $SpalteLeeresFeld)} {return}
	if {($Zeile == $ZeileLeeresFeld) || ($Spalte == $SpalteLeeresFeld)} {
		if {$Zeile == $ZeileLeeresFeld} {
			set tmp $Feld($Zeile,$SpalteLeeresFeld)
			if {$SpalteLeeresFeld>$Spalte} {
				for {set i $SpalteLeeresFeld} {$i>$Spalte} {incr i -1} {
					set Feld($Zeile,$i) $Feld($Zeile,[expr $i-1])
				}
			} else {
				for {set i $SpalteLeeresFeld} {$i<$Spalte} {incr i +1} {
					set Feld($Zeile,$i) $Feld($Zeile,[expr $i+1])
				}
			}
			set Feld($Zeile,$Spalte) $tmp
		} else {
			set tmp $Feld($ZeileLeeresFeld,$Spalte)
			if {$ZeileLeeresFeld>$Zeile} {
				for {set i $ZeileLeeresFeld} {$i>$Zeile} {incr i -1} {
					set Feld($i,$Spalte) $Feld([expr $i-1],$Spalte)
				}
			} else {
				for {set i $ZeileLeeresFeld} {$i<$Zeile} {incr i +1} {
					set Feld($i,$Spalte) $Feld([expr $i+1],$Spalte)
				}
			}
			set Feld($Zeile,$Spalte) $tmp
		}
		incr Puzzle(AnzahlZuege)
		BildAktualisieren 1
	}
}

proc PuzzleAufbauen {Neu} {
	# Erstellt ein neues Puzzle bzw. zeigt das zuletzt gespeicherte 
	# Puzzle an.
	#
	# Wenn Neu=1, dann wird ein neues Puzzle erstellt
	# wenn Neu=0, dann wird das zuvor gespeicherte Puzzle aufgebaut
	
	global InitialordnerBildOeffnen
	global Feld
	global Konf
	global Puzzle
	global ButtonNameLeeresFeld
	global Dlg

	if {$Neu == 1} {
		set PuzzleBreite $Konf(Breite)
		set Puzzle(ZeileMax) $Konf(ZeileMax)
		set Puzzle(SpalteMax) $Konf(SpalteMax)
		set Puzzle(ImageMagick) $Konf(ImageMagick)

		# Dialog, in dem der Anwender eine Bilddatei auswählen kann
		if {$Puzzle(ImageMagick) == 1} {
			set DateiExtension ".jpg .ppm"
			set DateiName [tk_getOpenFile -initialdir $InitialordnerBildOeffnen -filetypes {{Bilder ".jpg .ppm"} {Alle *}}]
		} else {
			set DateiExtension ".ppm"
			set DateiName [tk_getOpenFile -initialdir $InitialordnerBildOeffnen -filetypes {{Bilder ".ppm"} {Alle *}}]
			if {[string tolower [string range $DateiName end-3 end]] ne ".ppm"} {return}
		}
		if {$DateiName eq ""} {return}

		set OriginalBildName $DateiName
		
		# Aus der vom Anwender ausgewählten Bilddatei ein
		# ggf. verkleinertes Puzzlebild erstellen.
		# Dazu wird entweder ImageMagick genutzt (bessere Bildqualität)
		# oder die tcl-eigenen Befehle (schlechtere Bildqualität).
		if {$Puzzle(ImageMagick) == 1} {
			# Groesse aendern mit ImageMagick
			set OriginalBreite [exec identify -format "%w" $OriginalBildName]
			set OriginalHoehe [exec identify -format "%h" $OriginalBildName]
			# wenn keine Puzzlebreite im DialogSpielEinstellungen eingegeben wurde,
			# dann wird die Originlgröße verwendet.
			if {$PuzzleBreite eq ""} {set PuzzleBreite $OriginalBreite}
			set PuzzleHoehe [expr int(floor(1.0*$PuzzleBreite/$OriginalBreite*$OriginalHoehe))]
			#puts "Puzzle: $PuzzleBreite / $PuzzleHoehe"
			exec convert -resize ${PuzzleBreite}x$PuzzleHoehe $OriginalBildName $Puzzle(BildName)
			set PuzzleBild [image create photo -file $Puzzle(BildName)]
		} else {
			# Groesse aendern ohne ImageMagick (schlechtere Qualitaet, nur ganzzahlige Verhaeltnisse)
			set OriginalBild [image create photo -file $OriginalBildName]
			set OriginalBreite [image width $OriginalBild]
			set OriginalHoehe [image height $OriginalBild]
			# wenn keine Puzzlebreite im DialogSpielEinstellungen eingegeben wurde,
			# dann wird die Originlgröße verwendet.
			if {$PuzzleBreite eq ""} {set PuzzleBreite $OriginalBreite}
			set PuzzleBild [image create photo]
			set TmpImg [image create photo]
			set FaktorVergroessern [expr int(floor(10.0*$PuzzleBreite/$OriginalBreite))]
			set FaktorVerkleinern 10
			$TmpImg copy $OriginalBild -zoom $FaktorVergroessern
			$PuzzleBild copy $TmpImg -subsample $FaktorVerkleinern
			$PuzzleBild write $Puzzle(BildName)
			image delete $TmpImg
		}
	} else {
		set PuzzleBild [image create photo -file $Puzzle(BildName)]
	}
	set PuzzleBreite [image width $PuzzleBild]
	set PuzzleHoehe [image height $PuzzleBild]

	# Buttons und Frame löschen
	for {set Zeile 1} {$Zeile<=$Puzzle(ZeileMax)} {incr Zeile} {
		for {set Spalte 1} {$Spalte<=$Puzzle(SpalteMax)} {incr Spalte} {
			set ButtonName ${Zeile}_${Spalte}
			destroy .f1.b$ButtonName
		}
	}
	destroy .f1
	frame .f1 
	pack .f1

	set ButtonNameLeeresFeld ${Puzzle(ZeileMax)}_${Puzzle(SpalteMax)}
	set DeltaX [expr $PuzzleBreite/$Puzzle(SpalteMax)]
	set DeltaY [expr $PuzzleHoehe/$Puzzle(ZeileMax)]
	if {$Neu == 1} {
		unset Feld
		set Feld(1,1) ""
	}

	for {set Zeile 1} {$Zeile<=$Puzzle(ZeileMax)} {incr Zeile} {
		for {set Spalte 1} {$Spalte<=$Puzzle(SpalteMax)} {incr Spalte} {

			# ButtonName je Feld speichern
			set ButtonName ${Zeile}_${Spalte}
			if {$Neu == 1} {
				set Feld(${Zeile},${Spalte}) $ButtonName
			}

			# Puzzle-Teile erstellen
			set x1 [expr ($Spalte -1)*$DeltaX]
			set y1 [expr ($Zeile -1)*$DeltaY]
			set x2 [expr $Spalte*$DeltaX-1]
			set y2 [expr $Zeile*$DeltaY-1]
			#puts "$Zeile / $Spalte : $x1 / $y1 , $x2 / $y2"
			set img [image create photo]
			$img copy $PuzzleBild -from $x1 $y1 $x2 $y2
			
			if {($Zeile < $Puzzle(ZeileMax)) || ($Spalte < $Puzzle(SpalteMax))} {
				label .f1.b$ButtonName -image $img -borderwidth 1 -background black -foreground black -relief raise
			} else {
				label .f1.b$ButtonName -borderwidth 1 -background black -foreground black -relief raise
			}
			bind .f1.b$ButtonName <ButtonPress> {ButtonKlick %W}
			bind .f1.b$ButtonName <Enter> {%W config -background white}
			bind .f1.b$ButtonName <Leave> {%W config -background black}
		}
	}

	if {$Neu == 1} {
		# Zufaellige Startposition erzeugen
		set Richtung [expr int(2*rand())]
		for {set j 1} {$j<=$Konf(VerschiebenMax)} {incr j} {
			# zuerst das leere Feld ermitteln
			foreach i [array names Feld] {
				if {$Feld($i) eq $ButtonNameLeeresFeld} {
					set Trenner [string last , $i]
					set ZeileLeeresFeld [string range $i 0 [expr $Trenner-1]]
					set SpalteLeeresFeld [string range $i [expr $Trenner+1] end]
					#puts LeeresFeld,$ZeileLeeresFeld,$SpalteLeeresFeld,$Feld($i)
				}
			}
			if {$Richtung == 0} {
				# waagrecht
				set Zeile $ZeileLeeresFeld
				set Spalte $SpalteLeeresFeld
				while {$Spalte == $SpalteLeeresFeld} {
					set Spalte [expr 1+int(($Puzzle(SpalteMax))*rand())]
				}
				set Richtung 1
			} else {
				# senkrecht
				set Zeile $ZeileLeeresFeld
				while {$Zeile == $ZeileLeeresFeld} {
					set Zeile [expr 1+int(($Puzzle(ZeileMax))*rand())]
				}
				set Spalte $SpalteLeeresFeld
				set Richtung 0
			}
			set ButtonName ${Zeile}_${Spalte}
			ButtonKlick .b$ButtonName
		}
	} else {
		BildAktualisieren 0
	}		
	
	if {$Neu == 1} {
		set Puzzle(AnzahlZuege) 0
	}
	set Puzzle(Aktiv) 1
}

# **** Hauptprogramm ***********

SpracheEinstellen
KonfigurationEinlesen
SpracheEinstellen

set FensterX [expr {([winfo vrootwidth .]-400)/2}]
set FensterY [expr {([winfo vrootheight .]-400)/2}]
wm geometry . +${FensterX}+${FensterY}
wm title . [::msgcat::mc "%s" $ProgrammTitel]
wm minsize . 250 200
wm resizable . 0 0

frame .f2
pack .f2 -side bottom
label .f2.lbAnzahl -text [::msgcat::mc "Anzahl Züge:"]
label .f2.lbAnzahlZuege -textvariable Puzzle(AnzahlZuege)
pack .f2.lbAnzahl -side left
pack .f2.lbAnzahlZuege -side left

menu .mbar
. configure -menu .mbar
.mbar add cascade -label [::msgcat::mc "Datei"] -menu .mbar.datei -underline 0
.mbar add cascade -label [::msgcat::mc "Einstellungen"] -menu .mbar.einstellung -underline 0
.mbar add command -label [::msgcat::mc "Info"] -underline 0 -command {InfoAnzeigen $ProgrammName $ProgrammVersion}
menu .mbar.datei -tearoff 0
.mbar.datei add command -label [::msgcat::mc "Neu"] -underline 0 -command {PuzzleAufbauen 1}
.mbar.datei add command -label [::msgcat::mc "Beenden"] -underline 0 -command {ProgrammBeenden}
menu .mbar.einstellung -tearoff 0
.mbar.einstellung add cascade -label [::msgcat::mc "Sprache"] -menu .mbar.einstellung.sprache -underline 0
.mbar.einstellung add command -label [::msgcat::mc "Puzzle"] -underline 0 -command {DialogSpielEinstellungen}
menu .mbar.einstellung.sprache -tearoff 0
.mbar.einstellung.sprache add radiobutton -label "Deutsch" -variable Konf(Sprache) -value Deutsch -underline 0 -command {SpracheAendern}
.mbar.einstellung.sprache add radiobutton -label "English" -variable Konf(Sprache) -value English -underline 0 -command {SpracheAendern}
.mbar.einstellung.sprache add radiobutton -label "Espanol" -variable Konf(Sprache) -value Espanol -underline 1 -command {SpracheAendern}
.mbar.einstellung.sprache add radiobutton -label "Italiano" -variable Konf(Sprache) -value Italiano -underline 0 -command {SpracheAendern}
.mbar.einstellung.sprache add radiobutton -label "Türkce" -variable Konf(Sprache) -value Türkce -underline 0 -command {SpracheAendern}

SpielstandLaden
